@@ -44,6 +44,8 @@ shows the specified patch (defaulting to the current one)."""
 options = [make_option('-r', '--range',
                        metavar = 'rev1[..[rev2]]', dest = 'revs',
                        help = 'show the diff between revisions'),
+           make_option('-O', '--diff-opts',
+                       help = 'options to pass to git-diff'),
            make_option('-s', '--stat',
                        help = 'show the stat instead of the diff',
                        action = 'store_true')]
@@ -76,9 +78,15 @@ def func(parser, options, args):
         rev1 = 'HEAD'
         rev2 = None
 
-    if options.stat:
-        print git.diffstat(args, git_id(rev1), git_id(rev2))
+    if options.diff_opts:
+        diff_flags = options.diff_opts.split()
     else:
-        diff_str = git.diff(args, git_id(rev1), git_id(rev2))
+        diff_flags = []
+
+    if options.stat:
+        out.stdout_raw(git.diffstat(args, git_id(rev1), git_id(rev2)) + '\n')
+    else:
+        diff_str = git.diff(args, git_id(rev1), git_id(rev2),
+                            diff_flags = diff_flags )
         if diff_str:
             pager(diff_str)
