@@ -20,6 +20,7 @@ from optparse import OptionParser, make_option
 
 from stgit.commands.common import *
 from stgit.utils import *
+from stgit.config import GitConfigException
 from stgit import stack, git
 
 
@@ -41,18 +42,33 @@ options = [make_option('-n', '--nopush',
                        action = 'store_true'),
            make_option('-m', '--merged',
                        help = 'check for patches merged upstream',
+                       action = 'store_true'),
+           make_option('--force',
+                       help = 'force rebase even if the stack based was moved by (un)commits',
                        action = 'store_true')]
 
 def func(parser, options, args):
     """Pull the changes from a remote repository
     """
-    if len(args) > 1:
-        parser.error('incorrect number of arguments')
+    policy = config.get('branch.%s.stgit.pull-policy' % crt_series.get_name()) or \
+             config.get('stgit.pull-policy')
+
+    if policy == 'rebase':
+        # parent is local
+        if len(args) == 1:
+            parser.error('specifying a repository is meaningless for policy="%s"' % policy)
+        if len(args) > 0:
+            parser.error('incorrect number of arguments')
 
-    if len(args) >= 1:
-        repository = args[0]
     else:
-        repository = crt_series.get_parent_remote()
+        # parent is remote
+        if len(args) > 1:
+            parser.error('incorrect number of arguments')
+
+        if len(args) >= 1:
+            repository = args[0]
+        else:
+            repository = crt_series.get_parent_remote()
 
     if crt_series.get_protected():
         raise CmdException, 'This branch is protected. Pulls are not permitted'
@@ -61,13 +77,27 @@ def func(parser, options, args):
     check_conflicts()
     check_head_top_equal()
 
-    applied = prepare_rebase()
+    if policy == 'pull':
+        must_rebase = 0
+    elif policy == 'fetch-rebase':
+        must_rebase = 1
+    elif policy == 'rebase':
+        must_rebase = 1
+    else:
+        raise GitConfigException, 'Unsupported pull-policy "%s"' % policy
+
+    applied = prepare_rebase(force=options.force)
 
     # pull the remote changes
-    print 'Pulling from "%s"...' % repository
-    git.fetch(repository)
-    if (config.get('stgit.pull-does-rebase') == 'yes'):
+    if policy == 'pull':
+        out.info('Pulling from "%s"' % repository)
+        git.pull(repository)
+    elif policy == 'fetch-rebase':
+        out.info('Fetching from "%s"' % repository)
+        git.fetch(repository)
         rebase(git.fetch_head())
+    elif policy == 'rebase':
+        rebase(crt_series.get_parent_branch())
 
     post_rebase(applied, options.nopush, options.merged)
 
