@@ -30,7 +30,8 @@ class GitConfig:
         'stgit.smtpserver':	'localhost:25',
         'stgit.smtpdelay':	'5',
         'stgit.pullcmd':	'git-pull',
-        'stgit.pull-does-rebase': 'no',
+        'stgit.fetchcmd':	'git-fetch',
+        'stgit.pull-policy':	'pull',
         'stgit.merger':		'diff3 -L current -L ancestor -L patched -m -E ' \
 				'"%(branch1)s" "%(ancestor)s" "%(branch2)s" > "%(output)s"',
         'stgit.autoimerge':	'no',
@@ -98,9 +99,15 @@ class GitConfig:
 
     def rename_section(self, from_name, to_name):
         self.__run('git-repo-config --rename-section', [from_name, to_name])
+        self.__cache.clear()
 
     def set(self, name, value):
         self.__run('git-repo-config', [name, value])
+        self.__cache[name] = value
+
+    def unset(self, name):
+        self.__run('git-repo-config --unset', [name])
+        self.__cache[name] = None
 
     def sections_matching(self, regexp):
         """Takes a regexp with a single group, matches it against all
